package org.seasar.webwork.nazo;

import com.opensymphony.xwork.config.ConfigurationManager;
import com.opensymphony.xwork.config.RuntimeConfiguration;
import com.opensymphony.xwork.config.entities.ActionConfig;
import com.opensymphony.xwork.config.entities.PackageConfig;
import java.io.ByteArrayInputStream;
import java.util.Collections;
import javax.servlet.ServletContext;
import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.impl.S2ContainerImpl;

public class RuntimeConfigurationImplTest extends MockObjectTestCase {
    private S2Container container;
    private Mock servletContextMock;
    private Mock runtimeConfigurationMock;
    private RuntimeConfiguration runtimeConfiguration;

    public void setUp() {
	container = new S2ContainerImpl();
	servletContextMock = mock(ServletContext.class);
	container.setServletContext((ServletContext) servletContextMock.proxy());

	runtimeConfigurationMock = mock(RuntimeConfiguration.class);
	runtimeConfiguration = new RuntimeConfigurationImpl((RuntimeConfiguration) runtimeConfigurationMock.proxy(), container);
    }

    public void testGetActionConfigFromDelegate() {
	ActionConfig actionConfig = new ActionConfig();
	runtimeConfigurationMock.expects(once()).method("getActionConfig").with(eq("testNamespace"), eq("testName")).will(returnValue(actionConfig));
	assertSame(actionConfig, runtimeConfiguration.getActionConfig("testNamespace", "testName"));
    }

    public void testNazoPackageNotFound() {
	runtimeConfigurationMock.expects(once()).method("getActionConfig").with(eq("testNamespace"), eq("testName")).will(returnValue(null));
	assertNull(runtimeConfiguration.getActionConfig("testNamespace", "testName"));
    }

    public void testNamespaceMismatch() {
	PackageConfig packageConfig = new PackageConfig("nazo", "unknownNamespace", true, null);
	ConfigurationManager.getConfiguration().addPackageConfig(packageConfig.getName(), packageConfig);

	runtimeConfigurationMock.expects(once()).method("getActionConfig").with(eq("testNamespace"), eq("testName")).will(returnValue(null));
	assertNull(runtimeConfiguration.getActionConfig("testNamespace", "testName"));
    }

    public void testNazoPackageIsNotAbstract() {
	PackageConfig packageConfig = new PackageConfig("nazo", "testNamespace", false, null);
	ConfigurationManager.getConfiguration().addPackageConfig(packageConfig.getName(), packageConfig);

	runtimeConfigurationMock.expects(once()).method("getActionConfig").with(eq("testNamespace"), eq("testName")).will(returnValue(null));
	assertNull(runtimeConfiguration.getActionConfig("testNamespace", "testName"));
    }


    public void testGroovyScriptNotFound() {
 	PackageConfig packageConfig = new PackageConfig("nazo", "testNamespace", true, null);
 	ConfigurationManager.getConfiguration().addPackageConfig(packageConfig.getName(), packageConfig);

	runtimeConfigurationMock.expects(once()).method("getActionConfig").with(eq("testNamespace"), eq("testName")).will(returnValue(null));
	servletContextMock.expects(once()).method("getResourceAsStream").with(eq("/WEB-INF/nazo/testName.groovy")).will(returnValue(null));
	assertNull(runtimeConfiguration.getActionConfig("testNamespace", "testName"));
    }

    public void testGetGroovyActionConfig() {
 	PackageConfig defaultPackageConfig = ConfigurationManager.getConfiguration().getPackageConfig("webwork-default");
 	PackageConfig packageConfig = new PackageConfig("nazo", "testNamespace", true, null, Collections.singletonList(defaultPackageConfig));
	packageConfig.setDefaultInterceptorRef("completeStack");
	ConfigurationManager.getConfiguration().addPackageConfig(packageConfig.getName(), packageConfig);

	runtimeConfigurationMock.expects(once()).method("getActionConfig").with(eq("testNamespace"), eq("testName")).will(returnValue(null));
	servletContextMock.expects(once()).method("getResourceAsStream").with(eq("/WEB-INF/nazo/testName.groovy")).will(returnValue(new ByteArrayInputStream(new byte[] {})));

	ActionConfig actionConfig = runtimeConfiguration.getActionConfig("testNamespace", "testName");
	assertNotNull(actionConfig);
	assertEquals("/WEB-INF/nazo/testName.groovy", actionConfig.getClassName());
	assertNull(actionConfig.getExternalRefs());
	assertEquals(11, actionConfig.getInterceptors().size());
	assertNull(actionConfig.getMethodName());
	assertEquals("nazo", actionConfig.getPackageName());
	assertTrue(actionConfig.getParams().isEmpty());
	assertTrue(actionConfig.getResults() instanceof ResultConfigMap);
    }
}
