package org.seasar.webwork.nazo;

import com.opensymphony.webwork.config.Configuration;
import com.opensymphony.xwork.ObjectFactory;
import groovy.lang.GroovyObject;
import java.io.ByteArrayInputStream;
import javax.servlet.ServletContext;
import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.impl.S2ContainerImpl;

public class S2NazoObjectFactoryTest extends MockObjectTestCase {
    private S2Container container;
    private Mock servletContextMock;
    private ObjectFactory objectFactory;

    public void setUp() {
	container = new S2ContainerImpl();
	servletContextMock = mock(ServletContext.class);
	container.setServletContext((ServletContext) servletContextMock.proxy());

	objectFactory = new S2NazoObjectFactory(container);
    }

    public void testGetJavaClassInstance() throws ClassNotFoundException {
	assertEquals(String.class, objectFactory.getClassInstance("java.lang.String"));
    }

    public void testGetGroovyClassInstance() throws ClassNotFoundException {
	String testScript = "import com.opensymphony.xwork.Action\n"
	    + "\n"
	    + "class test implements Action {\n"
	    + "    String execute() {\n"
	    + "        return 'success'\n"
	    + "    }\n"
	    + "}";

	servletContextMock.expects(once()).method("getResourceAsStream").with(eq("/WEB-INF/nazo/test.groovy")).will(returnValue(new ByteArrayInputStream(testScript.getBytes())));
	assertEquals("test", objectFactory.getClassInstance("/WEB-INF/nazo/test.groovy").getName());
    }

    public void testI18N() throws Exception {
	String testScript = "import com.opensymphony.xwork.Action\n"
	    + "\n"
	    + "class test implements Action {\n"
	    + "    String helloWorld = 'ˤ'\n"
	    + "    String execute() {\n"
	    + "        return 'success'\n"
	    + "    }\n"
	    + "}";

	servletContextMock.expects(once()).method("getResourceAsStream").with(eq("/WEB-INF/nazo/test.groovy")).will(returnValue(new ByteArrayInputStream(testScript.getBytes())));
	Configuration.set("webwork.i18n.encoding", "EUC-JP");

 	final String fileEncoding = System.getProperty("file.encoding");
 	try {
	    System.setProperty("file.encoding", "US-ASCII");
	    Class clazz = objectFactory.getClassInstance("/WEB-INF/nazo/test.groovy");
	    GroovyObject groovyObject = (GroovyObject) objectFactory.buildBean(clazz);
	    assertEquals("ˤ", groovyObject.getProperty("helloWorld"));
 	} finally {
 	    System.setProperty("file.encoding", fileEncoding);
 	}
    }
}
